using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using CleverComponents.InetSuite;
using CleverComponents.InetSuite.Sspi;

namespace ServerSSL
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox edtPort;
		private System.Windows.Forms.Button btnStart;
		private System.Windows.Forms.Button btnStop;
		private System.Windows.Forms.TextBox memLog;
		private CleverComponents.InetSuite.CertificateStore certificateStore1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			server = new SslServer();

			//assigns event handlers for server
			server.GetCertificate += new GetCertificateEventHandler(server_GetCertificate);
			server.ConnectionAccepted += new ConnectionEventHandler(server_ConnectionAccepted);
			server.ConnectionClosed += new ConnectionEventHandler(server_ConnectionClosed);
			server.ConnectionRead += new ConnectionDataEventHandler(server_ConnectionRead);
			server.Started += new EventHandler(server_Started);
			server.Stopped += new EventHandler(server_Stopped);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.label1 = new System.Windows.Forms.Label();
			this.edtPort = new System.Windows.Forms.TextBox();
			this.btnStart = new System.Windows.Forms.Button();
			this.btnStop = new System.Windows.Forms.Button();
			this.label2 = new System.Windows.Forms.Label();
			this.memLog = new System.Windows.Forms.TextBox();
			this.certificateStore1 = new CleverComponents.InetSuite.CertificateStore();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Port";
			// 
			// edtPort
			// 
			this.edtPort.Location = new System.Drawing.Point(64, 16);
			this.edtPort.Name = "edtPort";
			this.edtPort.Size = new System.Drawing.Size(64, 20);
			this.edtPort.TabIndex = 1;
			this.edtPort.Text = "9002";
			// 
			// btnStart
			// 
			this.btnStart.Location = new System.Drawing.Point(160, 16);
			this.btnStart.Name = "btnStart";
			this.btnStart.TabIndex = 2;
			this.btnStart.Text = "Start";
			this.btnStart.Click += new System.EventHandler(this.btnStart_Click);
			// 
			// btnStop
			// 
			this.btnStop.Location = new System.Drawing.Point(248, 16);
			this.btnStop.Name = "btnStop";
			this.btnStop.TabIndex = 3;
			this.btnStop.Text = "Stop";
			this.btnStop.Click += new System.EventHandler(this.btnStop_Click);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 48);
			this.label2.Name = "label2";
			this.label2.TabIndex = 4;
			this.label2.Text = "Log";
			// 
			// memLog
			// 
			this.memLog.Location = new System.Drawing.Point(0, 72);
			this.memLog.Multiline = true;
			this.memLog.Name = "memLog";
			this.memLog.ReadOnly = true;
			this.memLog.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.memLog.Size = new System.Drawing.Size(344, 224);
			this.memLog.TabIndex = 5;
			this.memLog.Text = "";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(344, 294);
			this.Controls.Add(this.memLog);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.btnStop);
			this.Controls.Add(this.btnStart);
			this.Controls.Add(this.edtPort);
			this.Controls.Add(this.label1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Name = "Form1";
			this.Text = "SSL Server";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(new ExceptionHandler().OnThreadException);
			Application.Run(new Form1());
		}

		private SslServer server;

		private void btnStart_Click(object sender, System.EventArgs e) {
			server.Port = Convert.ToInt32(edtPort.Text);

			//forces the component to start SSL negotiation immediately once connecting
			server.UseTls = ServerTlsMode.Implicit;
			
			//specifies TLS 1.0 protocols (also available SSL 2.0 and SSL 3.0)
			server.TlsFlags = TlsFlags.UseTLS;

			//do not request client certificate for impersonation purposes
			server.RequireClientCertificate = false;

			server.Start();
		}

		private void btnStop_Click(object sender, System.EventArgs e) {
			server.Stop();
		}

		private void server_GetCertificate(object sender, GetCertificateEventArgs e) {
			//creates self-signed server certificate
			if(certificateStore1.Items.Count == 0) {
				certificateStore1.ValidFrom = DateTime.Now;
				certificateStore1.ValidTo = DateTime.Now.AddYears(1);
				Certificate cert = certificateStore1.CreateSelfSigned("CN=CleverTester,O=CleverComponents,E=CleverTester@company.mail", 0);
				certificateStore1.Items.Add(cert);
			}
			e.Certificate = certificateStore1.Items[0];
			e.Handled = true;
		}

		delegate void PutLogMessageDelegate(string logMessage);

		void PutLogMessage(string logMessage) {
			if (this.IsDisposed) return;

			if (this.InvokeRequired) {
				this.Invoke(new PutLogMessageDelegate(PutLogMessage), new object[] { logMessage });
			}
			else {
				memLog.Text += logMessage + "\r\n";
				memLog.Select(memLog.Text.Length, 0);
				memLog.ScrollToCaret();
			}
		}

		private void server_ConnectionAccepted(object sender, ConnectionEventArgs e) {
			PutLogMessage("Connected client: " + e.Connection.PeerIP);
		}

		void server_ConnectionClosed(object sender, ConnectionEventArgs e) {
			PutLogMessage("Disconnected client: " + e.Connection.PeerIP);
		}

		private void server_ConnectionRead(object sender, ConnectionDataEventArgs e) {
			PutLogMessage(String.Format("Received from {0} client: {1} bytes", e.Connection.PeerIP, e.Data.Length));
		}

		private void server_Started(object sender, EventArgs e) {
			PutLogMessage("=== Start ===");
		}

		private void server_Stopped(object sender, EventArgs e) {
			PutLogMessage("=== Stop");
		}
	}

	public class ExceptionHandler {
		public void OnThreadException(object sender, System.Threading.ThreadExceptionEventArgs t) {
			MessageBox.Show(t.Exception.Message);
		}
	}
}
